<?php

namespace Schaffen\BankIntegration\Cnab\Conciliacao\Cnab240\Banco;

use Schaffen\BankIntegration\Cnab\Conciliacao\Cnab240\AbstractRetorno;
use Schaffen\BankIntegration\Contracts\Boleto\Boleto as BoletoContract;
use Schaffen\BankIntegration\Contracts\Cnab\ConciliacaoCnab240;
use Schaffen\BankIntegration\Util;

class Itau extends AbstractRetorno implements ConciliacaoCnab240 
{
    /**
     * Código do banco
     *
     * @var string
     */
    protected $codigoBanco = BoletoContract::COD_BANCO_ITAU;

    /**
     * Array com as categorias.
     *
     * @var array
     */
    private $categorias = [
        //Débitos
        '101' => 'Cheques',
        '102' => 'Encargos',
        '103' => 'Estornos',
        '104' => 'Lançamentos avisados',
        '105' => 'Tarifas',
        '106' => 'Aplicação',
        '107' => 'Empréstimo / Financiamento',
        '108' => 'Cambio',
        '109' => 'CPMF',
        '110' => 'IOF',
        '111' => 'Imposto de renda',
        '112' => 'Pagamento fornecedores',
        '113' => 'Pagamento funcionários',
        '114' => 'Saque eletrônico',
        '115' => 'Ações',
        '116' => 'Seguro',
        '117' => 'Transferência entre contas',
        '118' => 'Devolução da compensação',
        '119' => 'Devolução de cheques',
        '120' => 'Transf. interbancárias (DOC/TED)',
        '121' => 'Desconto de duplicatas',
        //Créditos
        '201' => 'Depósitos',
        '202' => 'Cobrança',
        '203' => 'Devolução de cheques',
        '204' => 'Estornos',
        '205' => 'Lançamentos avisados',
        '206' => 'Resgate de aplicações',
        '207' => 'Empréstimo / Financiamento',
        '208' => 'Cambio',
        '209' => 'Transf. interbancárias (DOC/TED)',
        '210' => 'Ações',
        '211' => 'Dividendos',
        '212' => 'Seguro',
        '213' => 'Transferência entre contas',
        '214' => 'Depósitos especiais',
        '215' => 'Devolução da compensação',
    ];

    /**
     * Roda antes dos metodos de processar
     */
    // protected function init()
    // {
    //     $this->totais = [
    //         'liquidados' => 0,
    //         'entradas' => 0,
    //         'baixados' => 0,
    //         'protestados' => 0,
    //         'erros' => 0,
    //         'alterados' => 0,
    //     ];
    // }

    /**
     * @param array $header
     *
     * @return bool
     * @throws \Exception
     */
    protected function processarHeader(array $header)
    {
        $this->getHeader()
            ->setCodBanco($this->rem(1, 3, $header))
            ->setLoteServico($this->rem(4, 7, $header))
            ->setTipoRegistro($this->rem(8, 8, $header))
            ->setTipoInscricao($this->rem(18, 18, $header))
            ->setNumeroInscricao($this->rem(19, 32, $header))
            ->setConvenio($this->rem(48, 52, $header))
            ->setAgencia($this->rem(54, 57, $header))
            ->setAgenciaDv($this->rem(58, 58, $header))
            ->setConta($this->rem(66, 70, $header))
            ->setContaDv($this->rem(72, 72, $header))
            ->setNomeEmpresa($this->rem(73, 102, $header))
            ->setNomeBanco($this->rem(103, 132, $header))
            ->setCodigoRemessaRetorno($this->rem(143, 143, $header))
            ->setData($this->rem(144, 151, $header))
            ->setHora($this->rem(152, 157, $header))
            ->setNumeroSequencialArquivo($this->rem(158, 163, $header))
            ->setVersaoLayoutArquivo($this->rem(164, 166, $header));

        return true;
    }

    /**
     * @param array $headerLote
     *
     * @return bool
     * @throws \Exception
     */
    protected function processarHeaderLote(array $headerLote)
    {
        
        $this->getHeaderLote()
            ->setCodBanco($this->rem(1, 3, $headerLote))
            ->setNumeroLoteRetorno($this->rem(4, 7, $headerLote))
            ->setTipoRegistro($this->rem(8, 8, $headerLote))
            ->setTipoOperacao($this->rem(9, 9, $headerLote))
            ->setTipoServico($this->rem(10, 11, $headerLote))
            ->setVersaoLayoutLote($this->rem(14, 16, $headerLote))
            ->setTipoInscricao($this->rem(18, 18, $headerLote))
            ->setNumeroInscricao($this->rem(19, 32, $headerLote))
            ->setConvenio($this->rem(48, 52, $headerLote))
            ->setAgencia($this->rem(54, 57, $headerLote))
            ->setAgenciaDv($this->rem(58, 58, $headerLote))
            ->setConta($this->rem(66, 70, $headerLote))
            ->setContaDv($this->rem(72, 72, $headerLote))
            ->setNomeEmpresa($this->rem(73, 102, $headerLote))
            ->setDataInicial($this->rem(143, 150, $headerLote));

        return true;
    }

    /** 
     * @param array $detalhe
     *
     * @return bool
     * @throws \Exception
     */
    protected function processarDetalhe(array $detalhe){

        if ($this->getSegmentType($detalhe) == 'E') {
            $this->getDetalheConciliacao()
                ->setLinha($this->countLinha)
                ->setCodigoLote($this->rem(4, 7, $detalhe))
                ->setTipoRegistro($this->rem(8, 8, $detalhe))
                ->setNumeroRegistro($this->rem(9, 13, $detalhe)) 
                ->setSegmento($this->rem(14, 14, $detalhe))
                ->setIdLancamento($this->rem(15, 15, $detalhe))
                ->setTipoInscricao($this->rem(18, 18, $detalhe))
                ->setNumeroInscricao($this->rem(19, 32, $detalhe))
                ->setAgencia($this->rem(54, 57, $detalhe))
                ->setAgenciaDv($this->rem(58, 58, $detalhe))
                ->setConta($this->rem(66, 70, $detalhe))
                ->setContaDv($this->rem(72, 72, $detalhe))
                ->setNomeEmpresa($this->rem(73, 102, $detalhe))
                ->setNatureza($this->rem(109, 111, $detalhe))
                ->setDataLancamento($this->rem(143, 150, $detalhe))
                ->setValor(Util::nFloat($this->rem(151, 168, $detalhe)/100, 2, false))
                ->setTipoLancamento($this->rem(169, 169, $detalhe))
                ->setCategoria($this->rem(170, 172, $detalhe))
                ->setCategoriaDescricao(array_get($this->categorias, $this->getDetalheConciliacao()->getCategoria(), 'Desconhecida'))
                ->setHistorico($this->rem(177, 201, $detalhe))
                ->setNumeroDocumento($this->rem(235, 240, $detalhe))
                ->setStatusLinha(0);
        // ->setPagador([
        //     'nome' => $this->rem(149, 188, $detalhe),
        //     'documento' => $this->rem(134, 148, $detalhe),
        // ])
              
        }
      
        return true;
    }

    /**
     * @param array $trailer
     *
     * @return bool
     * @throws \Exception
     */
    protected function processarTrailerLote(array $trailer)
    {
        $this->getTrailerLote()
            ->setLoteServico($this->rem(4, 7, $trailer))
            ->setTipoRegistro($this->rem(8, 8, $trailer))
            ->setTipoInscricao($this->rem(18, 18, $trailer))
            ->setNumeroInscricao($this->rem(19, 32, $trailer))
            ->setDataFinal($this->rem(143, 150, $trailer))
            ->setQtdRegistroLote($this->rem(171, 176, $trailer));

        return true;
    }

    /**
     * @param array $trailer
     *
     * @return bool
     * @throws \Exception
     */
    protected function processarTrailer(array $trailer)
    {
        $this->getTrailer()
            ->setNumeroLote($this->rem(4, 7, $trailer))
            ->setTipoRegistro($this->rem(8, 8, $trailer))
            ->setQtdLotesArquivo((int) $this->rem(18, 23, $trailer))
            ->setQtdRegistroArquivo((int) $this->rem(24, 29, $trailer));

        return true;
    }
}
